//
//  InAppBrowser.h
//
//  Created by Andrew Winter on 2012/01/15.
//  Copyright (c) 2012 sbstrm. All rights reserved.
//
//  This software is licensed under the MIT/X11 Software License
//

#define TOOL_BAR_HEIGHT         44.0f

// List of all strings used
static NSString * const ACTION_CANCEL         = @"Cancel";
static NSString * const ACTION_OPEN_IN_SAFARI = @"Open in Safari";
static NSString * const ACTION_OPEN_IN_CHROME = @"Open in Chrome";
static NSString * const ACTION_COPY_URL       = @"Copy URL";


#import <UIKit/UIKit.h>
#import <QuartzCore/QuartzCore.h>

@interface InAppBrowser : UIViewController <UIWebViewDelegate, UIActionSheetDelegate, UISplitViewControllerDelegate>
{    
    // the current URL of the UIWebView
    NSURL *pageUrl;

    UIScrollView *webScrollView;
    
    // the UIWebView where we render the contents of the URL
    UIWebView *webPageView;
    
    // the UIToolbar with the "back" "forward" "reload" and "action" buttons
    UIToolbar *toolbar;
    NSTimer *hideToolBarTimer;
    
    // pointers to the buttons on the toolbar
    UIBarButtonItem *backButton;
    UIBarButtonItem *forwardButton;
    
    UIButton *reloadButton;
    UIButton *stopButton;
    
    UIView *addressView;
    UIImageView *progressView;
    UILabel *addressBar;
    
    BOOL toolbarHidden;
    BOOL firstLoadCalled;
    BOOL appActivityIndicatorShown;
    
    CGFloat progress;
    
    CAGradientLayer *gradient;
}

@property(nonatomic, retain) NSURL *pageUrl;

@property(nonatomic, retain) UIScrollView *webScrollView;

@property(nonatomic, retain) UIWebView *webPageView;
@property(nonatomic, retain) UIToolbar *toolbar;
@property(nonatomic, retain) NSTimer *hideToolBarTimer;
@property(nonatomic, retain) UIBarButtonItem *backButton;
@property(nonatomic, retain) UIBarButtonItem *forwardButton;

@property(nonatomic, retain) UIButton *reloadButton;
@property(nonatomic, retain) UIButton *stopButton;

@property(nonatomic, retain) UIView *addressView;
@property(nonatomic, retain) UIImageView *progressView;
@property(nonatomic, retain) UILabel *addressBar;

@property(nonatomic, retain) CAGradientLayer *gradient;

+ (InAppBrowser *)browserWithURLString:(NSString*)urlString;
+ (InAppBrowser *)browserWithURL:(NSURL*)url;

// Initializes the BrowserViewController with a specific URL 
- (id)initWithURLString:(NSString*)urlString;
- (id)initWithURL:(NSURL*)url;

- (void)setProgress:(CGFloat)aProgress;

- (void)setURLString:(NSString *)urlString;
- (void)setURL:(NSURL *)url;

@end
