//
//  InAppBrowser.m
//
//  Created by Andrew Winter on 2012/01/15.
//  Copyright (c) 2012 sbstrm. All rights reserved.
//
//  This software is licensed under the MIT/X11 Software License
//

#import "InAppBrowser.h"

@implementation InAppBrowser

@synthesize webPageView;
@synthesize pageUrl;
@synthesize toolbar;
@synthesize hideToolBarTimer;
@synthesize forwardButton;
@synthesize backButton;

@synthesize reloadButton;
@synthesize stopButton;

@synthesize webScrollView;

@synthesize addressView;
@synthesize progressView;
@synthesize addressBar;

@synthesize gradient;

/**********************************************************************************************************************/
#pragma mark - UIActionSheet Delegate

- (void)actionSheet:(UIActionSheet *)uias clickedButtonAtIndex:(NSInteger)buttonIndex
{
    // user pressed "Cancel"
    if(buttonIndex == [uias cancelButtonIndex]) return;
    
    // user pressed "Open in Safari"
    if([[uias buttonTitleAtIndex:buttonIndex] compare:IABLocalizedString(ACTION_OPEN_IN_SAFARI)] == NSOrderedSame)
    {
        [[UIApplication sharedApplication] openURL:pageUrl];
    }
    
    // user pressed "Open in Safari"
    if([[uias buttonTitleAtIndex:buttonIndex] compare:IABLocalizedString(ACTION_OPEN_IN_CHROME)] == NSOrderedSame)
    {
        NSString *scheme = pageUrl.scheme;
        
        // Replace the URL Scheme with the Chrome equivalent.
        NSString *chromeScheme = nil;
        if ([scheme isEqualToString:@"http"]) {
            chromeScheme = @"googlechrome";
        } else if ([scheme isEqualToString:@"https"]) {
            chromeScheme = @"googlechromes";
        }
        
        // Proceed only if a valid Google Chrome URI Scheme is available.
        if (chromeScheme) {
            NSString *absoluteString = [pageUrl absoluteString];
            NSRange rangeForScheme = [absoluteString rangeOfString:@":"];
            NSString *urlNoScheme =
            [absoluteString substringFromIndex:rangeForScheme.location];
            NSString *chromeURLString =
            [chromeScheme stringByAppendingString:urlNoScheme];
            NSURL *chromeURL = [NSURL URLWithString:chromeURLString];
            
            // Open the URL with Chrome.
            [[UIApplication sharedApplication] openURL:chromeURL];
        }else{
            [[UIApplication sharedApplication] openURL:pageUrl];
        }
    }
    
    if([[uias buttonTitleAtIndex:buttonIndex] compare:IABLocalizedString(ACTION_COPY_URL)] == NSOrderedSame)
    {
        UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
        pasteboard.string = [pageUrl absoluteString];
    }
}


/**********************************************************************************************************************/
#pragma mark - Object lifecycle

+ (InAppBrowser *)browserWithURLString:(NSString*)urlString{
    return [[[InAppBrowser alloc] initWithURLString:urlString] autorelease];
}
+ (InAppBrowser *)browserWithURL:(NSURL*)url{
    return [[[InAppBrowser alloc] initWithURL:url] autorelease];
}


- (void)dealloc
{
    [webPageView setDelegate:nil];
    [webPageView release];
    [toolbar release];
    [pageUrl release];
    
    [forwardButton release];
    [backButton release];
    
    [reloadButton release];
    [stopButton release];
    
    [addressView release];
    [progressView release];
    [addressBar release];

    [super dealloc];
}

- (id)initWithURLString:(NSString*)urlString{
    return [self initWithURL:[NSURL URLWithString:urlString]];
}

- (id)initWithURL:(NSURL*)url
{
    self = [self init];
    if(self){        
        self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        
        self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                                                                               target:self
                                                                                               action:@selector(actionButtonPressed:)] autorelease];
        
        toolbarHidden = NO;
        [self setToolbar:[[[UIToolbar alloc] initWithFrame:CGRectMake(0, 0, self.view.bounds.size.width, TOOL_BAR_HEIGHT)] autorelease]];
        toolbar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        [toolbar setTintColor:[UIColor darkGrayColor]];
        
        
        CGRect webFrame = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height);
        [self setWebPageView:[[[UIWebView alloc] initWithFrame:webFrame] autorelease]];
        webPageView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        [self.view addSubview:webPageView];
        
        [self.view addSubview:toolbar];
        
        [webPageView setDelegate:self];
        [webPageView setScalesPageToFit:YES];
        webPageView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        
        [self setURL:url];
        
        if([[[UIDevice currentDevice] systemVersion] compare:@"5.0" options:NSNumericSearch] == NSOrderedAscending){
            for (id subview in webPageView.subviews){
                if ([[subview class] isSubclassOfClass: [UIScrollView class]]){
                    webScrollView = ((UIScrollView *)subview);
                }
            }
        }else{
            webScrollView = [webPageView scrollView];
        }
        [webScrollView setContentInset:UIEdgeInsetsMake(TOOL_BAR_HEIGHT, 0, 0, 0)];
        
        
        [self setForwardButton:[[[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"right.png"] 
                                                                style:UIBarButtonItemStylePlain 
                                                               target:self 
                                                               action:@selector(forwardButtonPressed:)] autorelease]];
        
        
        [self setBackButton:[[[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"left.png"]
                                                             style:UIBarButtonItemStylePlain 
                                                            target:self 
                                                             action:@selector(backButtonPressed:)] autorelease]];
        
        [self setAddressView:[[[UIView alloc] initWithFrame:CGRectMake(80.0, 8.0, toolbar.bounds.size.width - 88.00, 28.0)] autorelease]];
        [addressView setBackgroundColor:[UIColor whiteColor]];
        addressView.layer.cornerRadius = 5.0f;
        addressView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        [self setProgressView:[[[UIImageView alloc] initWithFrame:addressView.bounds] autorelease]];
        progressView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        [self setAddressBar:[[[UILabel alloc] initWithFrame:CGRectMake(6.0, 0, progressView.bounds.size.width-addressView.bounds.size.height, progressView.bounds.size.height)] autorelease]];
        [addressBar setBackgroundColor:[UIColor clearColor]];
        [addressBar setFont:[UIFont fontWithName:@"HelveticaNeue" size:16.0f]];
        addressBar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        [progressView addSubview:addressBar];
        
        [addressView addSubview:progressView];
        
        [self setProgress:0.1f];
        
        [self setStopButton:[UIButton buttonWithType:UIButtonTypeCustom]];
        [stopButton setImage:[UIImage imageNamed:@"stop.png"] forState:UIControlStateNormal];
        [stopButton setFrame:CGRectMake(addressView.bounds.size.width-addressView.bounds.size.height, 0.0, addressView.bounds.size.height, addressView.bounds.size.height)];
        [stopButton addTarget:self action:@selector(stopButtonPressed:) forControlEvents:UIControlEventTouchUpInside];
        [addressView addSubview:stopButton];
        [stopButton setUserInteractionEnabled:YES];
        stopButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleBottomMargin;
        
        [self setReloadButton:[UIButton buttonWithType:UIButtonTypeCustom]];
        [reloadButton setImage:[UIImage imageNamed:@"refresh.png"] forState:UIControlStateNormal];
        [reloadButton setFrame:CGRectMake(addressView.bounds.size.width-addressView.bounds.size.height, 0.0, addressView.bounds.size.height, addressView.bounds.size.height)];
        [reloadButton addTarget:self action:@selector(reloadButtonPressed:) forControlEvents:UIControlEventTouchUpInside];
        [addressView addSubview:reloadButton];
        [reloadButton setEnabled:NO];
        [reloadButton setHidden:YES];
        reloadButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleBottomMargin;
        
        [toolbar addSubview:addressView];
    }
    return self;
}

- (void) viewWillAppear:(BOOL)animated{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
}

- (void)setURLString:(NSString *)urlString{
    [self setURL:[NSURL URLWithString:urlString]];
}

- (void)setURL:(NSURL *)url{
    [self setPageUrl:url];    
    [webPageView loadRequest:[NSURLRequest requestWithURL:pageUrl]];
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

/**********************************************************************************************************************/
#pragma mark - View lifecycle


- (void)updateToolbar
{
    // toolbar
    forwardButton.enabled = [webPageView canGoForward];
    backButton.enabled = [webPageView canGoBack];
    
    UIBarButtonItem *flexibleSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];   
    UIBarButtonItem *fixedButtonSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFixedSpace target:nil action:nil];       
    [fixedButtonSpace setWidth:10.0f]; 
    UIBarButtonItem *fixedSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFixedSpace target:nil action:nil];       
    [fixedSpace setWidth:addressView.bounds.size.width];
    
    NSMutableArray *toolbarButtons = [[NSMutableArray alloc] initWithObjects:backButton, fixedButtonSpace, forwardButton, fixedSpace, nil];
    [flexibleSpace release];
    [fixedSpace release];
    
    [toolbar setItems:toolbarButtons animated:YES];
    
    [toolbarButtons release];
    
    [addressBar setText:[pageUrl absoluteString]];
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    [self updateToolbar];
}


- (void)viewDidUnload
{
    if(hideToolBarTimer != nil){
        [hideToolBarTimer invalidate];
    }
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait
            || interfaceOrientation == UIInterfaceOrientationLandscapeRight
            || interfaceOrientation == UIInterfaceOrientationLandscapeLeft
            || interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown);
}


/**********************************************************************************************************************/
#pragma mark - User Interaction


- (void)backButtonPressed:(id)sender
{
    if([webPageView canGoBack]) [webPageView goBack];
}


- (void)forwardButtonPressed:(id)sender
{
    if([webPageView canGoForward]) [webPageView goForward];
}

- (void)stopButtonPressed:(id)sender{
    [webPageView stopLoading];
    [self updateToolbar];
    [self hideToolbarAnimation];
}

- (void)reloadButtonPressed:(id)sender{
    NSURLRequest *request = [NSURLRequest requestWithURL:pageUrl];
    [webPageView loadRequest:request];
    [self updateToolbar];
}


- (void)actionButtonPressed:(id)sender
{
    UIActionSheet *uias = nil;
    if([[UIApplication sharedApplication] canOpenURL:[NSURL URLWithString:@"googlechrome://"]]){
        uias = [[UIActionSheet alloc] initWithTitle:nil
                                                          delegate:self
                                                 cancelButtonTitle:IABLocalizedStringFormat(ACTION_CANCEL)
                                            destructiveButtonTitle:nil
                                                 otherButtonTitles:IABLocalizedStringFormat(ACTION_OPEN_IN_CHROME), IABLocalizedStringFormat(ACTION_OPEN_IN_SAFARI), IABLocalizedStringFormat(ACTION_COPY_URL), nil];
    }else{
        uias = [[UIActionSheet alloc] initWithTitle:nil
                                                          delegate:self
                                                 cancelButtonTitle:IABLocalizedStringFormat(ACTION_CANCEL)
                                            destructiveButtonTitle:nil
                                                 otherButtonTitles:IABLocalizedStringFormat(ACTION_OPEN_IN_SAFARI), IABLocalizedStringFormat(ACTION_COPY_URL), nil];
    }
    [uias showInView:self.view];
    [uias release];
}

-(void)showToolbar{
    firstLoadCalled = NO;
    [self setProgress:0.10f];
    [toolbar removeFromSuperview];
    
    toolbar.frame = CGRectMake(0, 0, toolbar.frame.size.width, toolbar.frame.size.height);
    [self.view addSubview:toolbar];
    toolbarHidden = NO;
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
}

-(void)hideToolbar{
    if(hideToolBarTimer != nil){
        [hideToolBarTimer invalidate];
    }
    [self setHideToolBarTimer:[NSTimer scheduledTimerWithTimeInterval:1.5f target:self selector:@selector(hideToolbarAnimation) userInfo:nil repeats:NO]];
}

-(void)hideToolbarAnimation{
    [self setProgress:0.0f];
    
    if(webScrollView != nil && !toolbarHidden){
        [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
        [self showStopButton:NO];
        BOOL animationEnabled = [UIView areAnimationsEnabled];
        [UIView setAnimationsEnabled:YES];
        toolbarHidden = YES;
        [UIView animateWithDuration:1.0f 
                              delay:0
                            options:UIViewAnimationOptionAllowUserInteraction
                         animations:^{
                             toolbar.frame = CGRectMake(0, -toolbar.frame.size.height, toolbar.frame.size.width, toolbar.frame.size.height);
                             if(webScrollView.contentOffset.y < 0){
                                 [webScrollView setContentOffset:CGPointMake(0, 0)];
                             }
                         }
                         completion:^(BOOL finished) {
                             [self setProgress:0.0f];
                             [toolbar removeFromSuperview];
                             
                             [webScrollView addSubview:toolbar];
                             
                             [UIView setAnimationsEnabled:animationEnabled];
                         }];
    }
}

- (void)setProgress:(CGFloat)aProgress {
    if (aProgress > 1.0 ) {
        return;
    }
    if([progressView.layer.sublayers count]){
        for(CALayer *layer in [[progressView.layer.sublayers copy] autorelease]){
            if([layer isKindOfClass:[CAGradientLayer class]]){
                [layer removeFromSuperlayer];
            }
        }
    }
    
    if(aProgress > 0.0){
        progress = aProgress;
        
        CGRect progressRect = CGRectZero;
        CGSize progressSize = CGSizeMake(progress * CGRectGetWidth(progressView.bounds), CGRectGetHeight(progressView.bounds));
        progressRect.size = progressSize;
                
        [self setGradient:[CAGradientLayer layer]];
        gradient.frame = progressRect;
        UIColor *top = [UIColor colorWithRed:115.0f/255.0f green:188.0f/255.0f blue:244.0f/255.0f alpha:0.9f];
        UIColor *middle = [UIColor colorWithRed:45.0f/255.0f green:144.0f/255.0f blue:241.0f/255.0f alpha:0.9f];
        gradient.colors = [NSArray arrayWithObjects:(id)[top CGColor], (id)[middle CGColor], (id)[top CGColor], nil];
        [progressView.layer insertSublayer:gradient atIndex:0];
        progressView.layer.cornerRadius = addressView.layer.cornerRadius;
        progressView.layer.masksToBounds = YES;
        progressView.layer.borderColor = [[UIColor darkGrayColor] CGColor];
        progressView.layer.borderWidth = 1.0;
    }
}

- (void)showStopButton:(BOOL)show{    
    if(show){
        [reloadButton setEnabled:NO];
        [reloadButton setHidden:YES];
        [stopButton setEnabled:YES];
        [stopButton setHidden:NO];
    }else{
        [stopButton setEnabled:NO];
        [stopButton setHidden:YES];
        [reloadButton setEnabled:YES];
        [reloadButton setHidden:NO];
    }
}

NSString* IABLocalizedStringFormat(NSString* key)
{
    static NSBundle* bundle = nil;
    if (nil == bundle) {
        NSString* path = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"InAppBrowser.bundle"];
        bundle = [[NSBundle bundleWithPath:path] retain];
    }
    return [bundle localizedStringForKey:key value:key table:nil];
}

NSString* IABLocalizedString(NSString* key, ...) 
{
	// Localize the format
	NSString *localizedStringFormat = IABLocalizedStringFormat(key);
    
	va_list args;
    va_start(args, key);
    NSString *string = [[[NSString alloc] initWithFormat:localizedStringFormat arguments:args] autorelease];
    va_end(args);
    
	return string;
}


/**********************************************************************************************************************/
#pragma mark - WebView Delegate


- (void)webViewDidStartLoad:(UIWebView *)webView{
    [self updateToolbar];
    
    if(toolbarHidden){
        [self showStopButton:YES];
        [self showToolbar];
    }
}

- (void)webViewDidFinishLoad:(UIWebView *)webView{
    
    if(!firstLoadCalled){
        [self setProgress:0.50f];
    }else{
        [self setProgress:(progress+0.02)];
    }
    firstLoadCalled = YES;
    
    if(![[pageUrl absoluteString] isEqualToString:[[[webView request] mainDocumentURL] absoluteString]]){
        [self setPageUrl:[[webView request] mainDocumentURL]];
        [self updateToolbar];
    }
    
    if(![webView isLoading]){
        [self hideToolbar];
    }
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{    
    [self updateToolbar];
}


#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    
}

@end
